import os
import boto3
import botocore
from botocore.client import Config
import re
import argparse
import glob
from ast import literal_eval
import sys
import shipyard_utils as shipyard
from shipyard_s3.cli import exit_codes as ec


def get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--source-file-name-match-type",
        dest="source_file_name_match_type",
        default="exact_match",
        choices={"exact_match", "regex_match"},
        required=False,
    )
    parser.add_argument("--source-file-name", dest="source_file_name", required=True)
    parser.add_argument(
        "--source-folder-name", dest="source_folder_name", default="", required=False
    )
    parser.add_argument(
        "--source-bucket-name", dest="source_bucket_name", default="", required=True
    )
    parser.add_argument(
        "--destination-folder-name",
        dest="destination_folder_name",
        default="",
        required=False,
    )
    parser.add_argument(
        "--destination-file-name",
        dest="destination_file_name",
        default=None,
        required=False,
    )
    parser.add_argument(
        "--destination-bucket-name",
        dest="destination_bucket_name",
        default="",
        required=True,
    )
    parser.add_argument("--aws-access-key-id", dest="aws_access_key_id", required=False)
    parser.add_argument(
        "--aws-secret-access-key", dest="aws_secret_access_key", required=False
    )
    parser.add_argument(
        "--aws-default-region", dest="aws_default_region", required=False
    )

    return parser.parse_args()


def set_environment_variables(args):
    """
    Set AWS credentials as environment variables if they're provided via keyword arguments
    rather than seeded as environment variables. This will override system defaults.
    """

    if args.aws_access_key_id:
        os.environ["AWS_ACCESS_KEY_ID"] = args.aws_access_key_id
    if args.aws_secret_access_key:
        os.environ["AWS_SECRET_ACCESS_KEY"] = args.aws_secret_access_key
    if args.aws_default_region:
        os.environ["AWS_DEFAULT_REGION"] = args.aws_default_region
    return


def s3_list_files(
    s3_connection,
    bucket_name,
    source_folder,
):
    """List files in s3"""
    try:
        # s3_response = s3_connection.list_objects_v2(Bucket=bucket_name, Prefix=source_folder)
        # files_list =  [
        #     _file['Key'] for _file in s3_response['Contents']
        # ]
        bucket = s3_connection.Bucket(bucket_name)

        files_list = [obj.key for obj in bucket.objects.filter(Prefix=source_folder)]
        return files_list
    except:
        print(
            f"There was an error locating the files. Either the bucket does not exist or the folder does not exist. Please ensure that both are correct."
        )
        sys.exit(ec.EXIT_CODE_FILE_NOT_FOUND)


def connect_to_s3(access_key_id, secret_access_key, default_region=None):
    """
    Create a connection to the S3 service using credentials provided as environment variables.
    """
    try:
        session = boto3.Session(
            aws_access_key_id=access_key_id,
            aws_secret_access_key=secret_access_key,
            region_name=default_region,
        )

        s3_connection = session.resource("s3")
        return s3_connection
    except Exception as e:
        print(
            "Error: Could not connect to S3. Ensure that the provided access key, secret key, and region are correct"
        )
        print(e)
        sys.exit(ec.EXIT_CODE_INVALID_CREDENTIALS)


def move_s3_file(
    s3_connection,
    source_bucket_name,
    destination_bucket_name,
    source_full_path,
    destination_full_path,
):
    """
    Moves an AWS S3 file from one bucket to another.

    The specific way it does this is by first copying the file from one bucket to another
    then deleting the file in the source_bucket.
    """
    # create a source dictionary that specifies bucket name and key name of the object to be copied
    copy_source = {"Bucket": source_bucket_name, "Key": source_full_path}

    bucket = s3_connection.Bucket(destination_bucket_name)
    try:
        bucket.copy(copy_source, destination_full_path)

        s3_connection.Object(source_bucket_name, source_full_path).delete()

        print(
            f"{source_full_path} successfully moved to {destination_bucket_name}/{destination_full_path}"
        )
    except Exception as e:
        print(f"An error occured {e}.")
        print(f"The file {source_bucket_name}/{source_full_path} could not be found")
        sys.exit(ec.EXIT_CODE_FILE_NOT_FOUND)


def main():
    args = get_args()
    set_environment_variables(args)
    source_file_name = args.source_file_name
    source_folder_name = args.source_folder_name
    source_full_path = shipyard.files.combine_folder_and_file_name(
        source_folder_name, source_file_name
    )
    destination_folder_name = shipyard.files.clean_folder_name(
        args.destination_folder_name
    )
    source_file_name_match_type = args.source_file_name_match_type
    # get arguments from OS environment
    aws_access_key_id = os.environ["AWS_ACCESS_KEY_ID"]
    aws_secret_access_key = os.environ["AWS_SECRET_ACCESS_KEY"]
    aws_default_region = os.environ["AWS_DEFAULT_REGION"]
    source_bucket_name = args.source_bucket_name
    destination_bucket_name = args.destination_bucket_name

    s3_connection = connect_to_s3(
        aws_access_key_id, aws_secret_access_key, aws_default_region
    )

    if source_file_name_match_type == "regex_match":
        file_names = s3_list_files(
            s3_connection, source_bucket_name, source_folder_name
        )
        ## exit if there is a regex error
        try:
            matching_file_names = shipyard.files.find_all_file_matches(
                file_names, source_file_name
            )
            num_matches = len(matching_file_names)
        except Exception as e:
            print(
                f"Error in finding regex matches. Please make sure a valid regex is entered"
            )
            sys.exit(ec.EXIT_CODE_INVALID_REGEX)

        if num_matches == 0:
            print(f"No matches found for regex {source_file_name}")
            sys.exit(1)
        else:
            print(f"{num_matches} files found. Preparing to upload...")

        try:
            for index, key_name in enumerate(matching_file_names, 1):
                destination_full_path = shipyard.files.determine_destination_full_path(
                    destination_folder_name=destination_folder_name,
                    destination_file_name=args.destination_file_name,
                    source_full_path=key_name,
                    file_number=None if num_matches == 1 else index,
                )
                print(f"Moving file {index} of {len(matching_file_names)}")
                move_s3_file(
                    s3_connection,
                    source_bucket_name,
                    destination_bucket_name,
                    key_name,
                    destination_full_path,
                )
        except Exception as e:
            print("Something went wrong moving the files")
            print(e)
            sys.exit(1)

    else:
        destination_file_name = shipyard.files.determine_destination_file_name(
            source_full_path=source_full_path,
            destination_file_name=args.destination_file_name,
        )
        # destination_file_name = args.destination_file_name
        destination_full_path = shipyard.files.determine_destination_full_path(
            destination_folder_name=destination_folder_name,
            destination_file_name=destination_file_name,
            source_full_path=source_full_path,
        )

        move_s3_file(
            s3_connection,
            source_bucket_name,
            destination_bucket_name,
            source_full_path,
            destination_full_path,
        )


if __name__ == "__main__":
    main()



import os
import boto3
import botocore
from botocore.client import Config
import re
import argparse
import glob
from ast import literal_eval
import sys
import shipyard_utils as shipyard
from shipyard_s3.cli import exit_codes as ec


def get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("--bucket-name", dest="bucket_name", required=True)
    parser.add_argument(
        "--source-file-name-match-type",
        dest="source_file_name_match_type",
        default="exact_match",
        choices={"exact_match", "regex_match"},
        required=False,
    )
    parser.add_argument("--source-file-name", dest="source_file_name", required=True)
    parser.add_argument(
        "--source-folder-name", dest="source_folder_name", default="", required=False
    )
    parser.add_argument("--s3-config", dest="s3_config", default=None, required=False)
    parser.add_argument("--aws-access-key-id", dest="aws_access_key_id", required=False)
    parser.add_argument(
        "--aws-secret-access-key", dest="aws_secret_access_key", required=False
    )
    parser.add_argument(
        "--aws-default-region", dest="aws_default_region", required=False
    )
    return parser.parse_args()


def set_environment_variables(args):
    """
    Set AWS credentials as environment variables if they're provided via keyword arguments
    rather than seeded as environment variables. This will override system defaults.
    """

    if args.aws_access_key_id:
        os.environ["AWS_ACCESS_KEY_ID"] = args.aws_access_key_id
    if args.aws_secret_access_key:
        os.environ["AWS_SECRET_ACCESS_KEY"] = args.aws_secret_access_key
    if args.aws_default_region:
        os.environ["AWS_DEFAULT_REGION"] = args.aws_default_region
    return


def connect_to_s3(s3_config=None):
    """
    Create a connection to the S3 service using credentials provided as environment variables.
    """
    s3_connection = boto3.client("s3", config=Config(s3_config))
    return s3_connection


def s3_list_files(
    s3_connection,
    bucket_name,
    source_folder,
):
    """List files in s3"""
    s3_response = s3_connection.list_objects_v2(
        Bucket=bucket_name, Prefix=source_folder
    )
    files_list = [_file["Key"] for _file in s3_response["Contents"]]
    return files_list


def remove_s3_file(
    s3_connection,
    bucket_name,
    source_full_path,
):
    """
    Uploads a single file to S3. Uses the s3.transfer method to ensure that files larger than 5GB are split up during the upload process.

    Extra Args can be found at https://boto3.amazonaws.com/v1/documentation/api/latest/guide/s3-uploading-files.html#the-extraargs-parameter
    and are commonly used for custom file encryption or permissions.
    """
    try:
        s3_response = s3_connection.delete_object(
            Bucket=bucket_name, Key=source_full_path
        )

        print(f"{source_full_path} delete function successful")
    except Exception as e:
        print(f"Error: {source_full_path} not found in bucket {bucket_name}.")
        sys.exit(ec.EXIT_CODE_FILE_NOT_FOUND)


def main():
    args = get_args()
    set_environment_variables(args)
    bucket_name = args.bucket_name
    source_file_name = args.source_file_name
    source_folder_name = shipyard.files.clean_folder_name(args.source_folder_name)
    source_full_path = shipyard.files.combine_folder_and_file_name(
        source_folder_name, source_file_name
    )
    source_file_name_match_type = args.source_file_name_match_type
    s3_config = args.s3_config

    s3_connection = connect_to_s3(s3_config)
    if source_file_name_match_type == "regex_match":
        file_names = s3_list_files(s3_connection, bucket_name, source_folder_name)
        ## exit if there is a regex error
        try:
            matching_file_names = shipyard.files.find_all_file_matches(
                file_names, source_file_name
            )
            num_matches = len(matching_file_names)
        except Exception as e:
            print(
                f"Error in finding regex matches. Please make sure a valid regex is entered"
            )
            sys.exit(ec.EXIT_CODE_INVALID_REGEX)

        if num_matches == 0:
            print(f"No matches found for regex {source_file_name}")
            sys.exit(1)
        else:
            print(f"{num_matches} files found. Preparing to remove...")

        for index, key_name in enumerate(matching_file_names, 1):
            remove_s3_file(
                source_full_path=key_name,
                bucket_name=bucket_name,
                s3_connection=s3_connection,
            )
            print(f"Removing file {index} of {len(matching_file_names)}")

    else:
        remove_s3_file(
            source_full_path=source_full_path,
            bucket_name=bucket_name,
            s3_connection=s3_connection,
        )


if __name__ == "__main__":
    main()



import os
import boto3
import botocore
from botocore.client import Config
import re
import argparse
import glob
from ast import literal_eval
import sys


def get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("--bucket-name", dest="bucket_name", required=True)
    parser.add_argument(
        "--source-file-name-match-type",
        dest="source_file_name_match_type",
        default="exact_match",
        choices={"exact_match", "regex_match"},
        required=False,
    )
    parser.add_argument("--source-file-name", dest="source_file_name", required=True)
    parser.add_argument(
        "--source-folder-name", dest="source_folder_name", default="", required=False
    )
    parser.add_argument(
        "--destination-folder-name",
        dest="destination_folder_name",
        default="",
        required=False,
    )
    parser.add_argument(
        "--destination-file-name",
        dest="destination_file_name",
        default=None,
        required=False,
    )
    parser.add_argument("--s3-config", dest="s3_config", default=None, required=False)
    parser.add_argument("--aws-access-key-id", dest="aws_access_key_id", required=False)
    parser.add_argument(
        "--aws-secret-access-key", dest="aws_secret_access_key", required=False
    )
    parser.add_argument(
        "--aws-default-region", dest="aws_default_region", required=False
    )
    parser.add_argument("--extra-args", dest="extra_args", required=False)
    return parser.parse_args()


def set_environment_variables(args):
    """
    Set AWS credentials as environment variables if they're provided via keyword arguments
    rather than seeded as environment variables. This will override system defaults.
    """

    if args.aws_access_key_id:
        os.environ["AWS_ACCESS_KEY_ID"] = args.aws_access_key_id
    if args.aws_secret_access_key:
        os.environ["AWS_SECRET_ACCESS_KEY"] = args.aws_secret_access_key
    if args.aws_default_region:
        os.environ["AWS_DEFAULT_REGION"] = args.aws_default_region
    return


def connect_to_s3(s3_config=None):
    """
    Create a connection to the S3 service using credentials provided as environment variables.
    """
    s3_connection = boto3.client("s3", config=Config(s3_config))
    return s3_connection


def extract_file_name_from_source_full_path(source_full_path):
    """
    Use the file name provided in the source_full_path variable. Should be run only
    if a destination_file_name is not provided.
    """
    destination_file_name = os.path.basename(source_full_path)
    return destination_file_name


def enumerate_destination_file_name(destination_file_name, file_number=1):
    """
    Append a number to the end of the provided destination file name.
    Only used when multiple files are matched to, preventing the destination file from being continuously overwritten.
    """
    if re.search(r"\.", destination_file_name):
        destination_file_name = re.sub(
            r"\.", f"_{file_number}.", destination_file_name, 1
        )
    else:
        destination_file_name = f"{destination_file_name}_{file_number}"
    return destination_file_name


def determine_destination_file_name(
    *, source_full_path, destination_file_name, file_number=None
):
    """
    Determine if the destination_file_name was provided, or should be extracted from the source_file_name,
    or should be enumerated for multiple file downloads.
    """
    if destination_file_name:
        if file_number:
            destination_file_name = enumerate_destination_file_name(
                destination_file_name, file_number
            )
        else:
            destination_file_name = destination_file_name
    else:
        destination_file_name = extract_file_name_from_source_full_path(
            source_full_path
        )

    return destination_file_name


def clean_folder_name(folder_name):
    """
    Cleans folders name by removing duplicate '/' as well as leading and trailing '/' characters.
    """
    folder_name = folder_name.strip("/")
    if folder_name != "":
        folder_name = os.path.normpath(folder_name)
    return folder_name


def combine_folder_and_file_name(folder_name, file_name):
    """
    Combine together the provided folder_name and file_name into one path variable.
    """
    combined_name = os.path.normpath(
        f'{folder_name}{"/" if folder_name else ""}{file_name}'
    )
    combined_name = os.path.normpath(combined_name)

    return combined_name


def determine_destination_full_path(
    destination_folder_name, destination_file_name, source_full_path, file_number=None
):
    """
    Determine the final destination name of the file being downloaded.
    """
    destination_file_name = determine_destination_file_name(
        destination_file_name=destination_file_name,
        source_full_path=source_full_path,
        file_number=file_number,
    )
    destination_full_path = combine_folder_and_file_name(
        destination_folder_name, destination_file_name
    )
    return destination_full_path


def find_all_local_file_names(source_folder_name):
    """
    Returns a list of all files that exist in the current working directory,
    filtered by source_folder_name if provided.
    """
    cwd = os.getcwd()
    cwd_extension = os.path.normpath(f"{cwd}/{source_folder_name}/**")
    file_names = glob.glob(cwd_extension, recursive=True)
    return file_names


def find_all_file_matches(file_names, file_name_re):
    """
    Return a list of all file_names that matched the regular expression.
    """
    matching_file_names = []
    for file in file_names:
        if re.search(file_name_re, file):
            matching_file_names.append(file)

    return matching_file_names


def upload_s3_file(
    s3_connection, bucket_name, source_full_path, destination_full_path, extra_args=None
):
    """
    Uploads a single file to S3. Uses the s3.transfer method to ensure that files larger than 5GB are split up during the upload process.

    Extra Args can be found at https://boto3.amazonaws.com/v1/documentation/api/latest/guide/s3-uploading-files.html#the-extraargs-parameter
    and are commonly used for custom file encryption or permissions.
    """
    s3_upload_config = boto3.s3.transfer.TransferConfig()
    s3_transfer = boto3.s3.transfer.S3Transfer(
        client=s3_connection, config=s3_upload_config
    )

    s3_transfer.upload_file(
        source_full_path, bucket_name, destination_full_path, extra_args=extra_args
    )

    print(
        f"{source_full_path} successfully uploaded to {bucket_name}/{destination_full_path}"
    )


def main():
    args = get_args()
    set_environment_variables(args)
    bucket_name = args.bucket_name
    source_file_name = args.source_file_name
    source_folder_name = args.source_folder_name
    source_full_path = combine_folder_and_file_name(
        folder_name=f"{os.getcwd()}/{source_folder_name}", file_name=source_file_name
    )
    destination_folder_name = clean_folder_name(args.destination_folder_name)
    source_file_name_match_type = args.source_file_name_match_type
    s3_config = args.s3_config
    extra_args = literal_eval(args.extra_args if args.extra_args else "{}")

    s3_connection = connect_to_s3(s3_config)

    if source_file_name_match_type == "regex_match":
        file_names = find_all_local_file_names(source_folder_name)
        matching_file_names = find_all_file_matches(
            file_names, re.compile(source_file_name)
        )
        num_matches = len(matching_file_names)

        if num_matches == 0:
            print(f"No matches found for regex {source_file_name}")
            sys.exit(1)
        else:
            print(f"{num_matches} files found. Preparing to upload...")

        for index, key_name in enumerate(matching_file_names):
            destination_full_path = determine_destination_full_path(
                destination_folder_name=destination_folder_name,
                destination_file_name=args.destination_file_name,
                source_full_path=key_name,
                file_number=None if num_matches == 1 else index + 1,
            )
            print(f"Uploading file {index+1} of {len(matching_file_names)}")
            upload_s3_file(
                source_full_path=key_name,
                destination_full_path=destination_full_path,
                bucket_name=bucket_name,
                extra_args=extra_args,
                s3_connection=s3_connection,
            )

    else:
        destination_full_path = determine_destination_full_path(
            destination_folder_name=destination_folder_name,
            destination_file_name=args.destination_file_name,
            source_full_path=source_full_path,
        )
        upload_s3_file(
            source_full_path=source_full_path,
            destination_full_path=destination_full_path,
            bucket_name=bucket_name,
            extra_args=extra_args,
            s3_connection=s3_connection,
        )


if __name__ == "__main__":
    main()



import os
import sys
import boto3
from shipyard_blueprints import S3Client


def main():
    s3 = S3Client(
        aws_access_key=os.getenv("AWS_ACCESS_KEY_ID"),
        aws_secret_access_key=os.getenv("AWS_SECRET_ACCESS_KEY"),
    )  # only necessary for logging
    try:
        client = boto3.client(
            "sts",
            aws_access_key_id=os.getenv("AWS_ACCESS_KEY_ID"),
            aws_secret_access_key=os.getenv("AWS_SECRET_ACCESS_KEY"),
        )
        response = client.get_caller_identity()
        if response["ResponseMetadata"]["HTTPStatusCode"] == 200:
            s3.logger.info("Successfully connected to S3")
            sys.exit(0)
    except Exception as e:
        s3.logger.error(f"Could not connect to the AWS S3")
        sys.exit(1)

    else:
        s3.logger.error(f"Could not connect to the AWS S3")
        sys.exit(1)


if __name__ == "__main__":
    main()



from shipyard_templates import CloudStorage
import boto3


class S3Client(CloudStorage):
    def __init__(
        self, aws_access_key: str, aws_secret_access_key: str, region: str = None
    ) -> None:
        self.aws_access_key = aws_access_key
        self.aws_secret_access_key = aws_secret_access_key
        self.region = region
        super().__init__(
            aws_access_key=aws_access_key,
            aws_secret_access_key=aws_secret_access_key,
            region=region,
        )

    def connect(self):
        client = boto3.client(
            "s3",
            region_name=self.region,
            aws_access_key_id=self.aws_access_key,
            aws_secret_access_key=self.aws_secret_access_key,
            config=None,
        )
        self.logger.info("Successfully connected to S3")
        return client

    def move_or_rename_files(self):
        pass

    def remove_files(self):
        pass

    def upload_files(self):
        pass

    def download_files(self):
        pass
